@@ -17,12 +17,13 @@
 
 import os
 import json
-from mongoengine import Document, EmbeddedDocument
+from mongoengine import Document, EmbeddedDocument, QuerySetManager, QuerySet
 from tornado.web import RequestHandler, HTTPError
 from tornado.web import StaticFileHandler as StaticFileHandlerTornado
 from pyrocumulus.converters import get_converter, get_request_converter
 from pyrocumulus.parsers import get_parser
-from pyrocumulus.exceptions import PyrocumulusException, StaticFileError
+from pyrocumulus.exceptions import (PyrocumulusException, StaticFileError,
+                                    PyrocumulusConfusionError)
 from pyrocumulus.web.template import render_template
 
 
@@ -225,9 +226,11 @@ class RestHandler(ModelHandler):
 
         :param method_to_call: callable which receives self.params as params.
         """
+        # propably this whole thing of call_method_and_write_json is an idiot
+        # idea. I need to change it in future. Don't rely on it.
+        # and yeah, this code is crappy. Sorry.
         returned_obj = method_to_call(**self.params)
-        converter = get_converter(returned_obj, max_depth=self.object_depth)
-        mydict = converter.sanitize_dict(converter.to_dict())
+        mydict = self._get_clean_dict(returned_obj)
         mydict.update(self.json_extra_params)
         self.write(mydict)
 
@@ -325,6 +328,33 @@ class RestHandler(ModelHandler):
             raise HTTPError(405)
         return self.operations[operation]
 
+    def _get_clean_dict(self, obj):
+        """
+        Returns a dict ready to serialize. Use pyrocumulus.converters
+        to do that.
+        """
+        if (isinstance(obj, Document)
+            or isinstance(obj, EmbeddedDocument)
+            or isinstance(obj, QuerySetManager)
+            or isinstance(obj, QuerySet)):
+
+            converter = get_converter(obj, max_depth=self.object_depth)
+            mydict = converter.sanitize_dict(converter.to_dict())
+
+        elif isinstance(obj, dict):
+            mydict = obj
+
+        elif isinstance(obj, list):
+            mylist = [self._get_clean_dict(o) for o in obj]
+            mydict = {'items': mylist,
+                      'quantity': len(obj)}
+        else:
+            raise PyrocumulusConfusionError(
+                'I\'m confused. I don\'t know what to do with %s' % str(obj))
+
+        return mydict
+
+
     def _get_jsonp(self, chunk, callback):
         mystr = str(chunk)
         jsonp = '%s(%s)' % (callback, mystr)
